@@ -297,7 +297,11 @@ namespace pan
           local GroupHeaders.  As a side-effect, the value of `part'
           after this call is undefined.  This is an ugly interface,
           but it's fast and only called by one client. */
-      void load_part (const Quark& g, const Quark& mid, size_t number, size_t lines, Article::Part& part);
+      void load_part (const Quark& g, const Quark& mid,
+                      int number,
+                      const StringView& part_mid,
+                      unsigned long lines,
+                      unsigned long bytes);
 
       void load_headers (const DataIO&, const Quark& group);
       void save_headers (DataIO&, const Quark& group) const;
@@ -515,7 +519,7 @@ namespace pan
        */
       struct XOverEntry
       {
-        int _batch_parts_size;
+        time_t _last_flush_time;
 
         /** These are the articles which have been recently added.
             The patch is periodically flushed to on_articles_added()
@@ -539,7 +543,7 @@ namespace pan
             be assigned to the same XOVER task. */
         int refcount;
 
-        XOverEntry(): _batch_parts_size(0), refcount(0) { }
+        XOverEntry(): _last_flush_time(0), refcount(0) { }
       };
 
       typedef Loki::AssocVector<Quark,XOverEntry> xovers_t;
