@@ -23,8 +23,8 @@
 #include <pan/general/debug.h>
 #include <pan/general/foreach.h>
 #include <pan/general/quark.h>
+#include <pan/usenet-utils/filter-info.h>
 #include <pan/data/article.h>
-#include <pan/data/filter-info.h>
 #include "article-filter.h"
 #include "data-impl.h"
 
@@ -370,7 +370,7 @@ DataImpl :: MyTree :: add_articles (const const_nodes_v& nodes_in)
   ///  1. add the new articles
   ///
 
-  // sort the nodes by Message-Id
+  // sort the nodes by Message-ID
   // so that we can use `nodes' for set operations
   const_nodes_v nodes (nodes_in);
   std::sort (nodes.begin(), nodes.end(), compare);
