@@ -103,6 +103,7 @@ namespace pan
 
          bool operator== (const StringView& p) const { return !strcmp(p); }
          bool operator!= (const StringView& p) const { return !(*this == p); }
+         bool operator<  (const StringView& p) const { return strcmp(p)<0; }
 
          bool operator== (const char *s) const { return !strcmp(str,len,s,strlen(s)); }
          bool operator!= (const char *s) const { return  strcmp(str,len,s,strlen(s)); }
@@ -125,7 +126,7 @@ namespace pan
             return strcmp (str, len, s, l); }
 
          char* strchr (char needle, size_t p=0) const {
-            return p<len ? strchr (str+p, len, needle) : 0; }
+            return p<len ? strchr (str+p, len-p, needle) : 0; }
 
          char* strrchr (char needle) const {
             return strrchr (str, len, needle); }
