@@ -36,6 +36,7 @@ extern "C" {
 #include <pan/icons/pan-pixbufs.h>
 #include "actions.h"
 #include "body-pane.h"
+#include "e-charset-picker.h"
 #include "dl-headers-ui.h"
 #include "group-pane.h"
 #include "group-prefs-dialog.h"
@@ -182,17 +183,8 @@ GUI :: GUI (Data& data, Queue& queue, ArticleCache& cache, Prefs& prefs, GroupPr
   g_object_weak_ref (G_OBJECT(_root), (GWeakNotify)g_object_unref, _ttips);
 
   char * filename = g_build_filename (file::get_pan_home().c_str(), "pan.ui", NULL);
-#if 0
-  GError * gerr (0);
-  gtk_ui_manager_add_ui_from_file (_ui_manager, filename, &gerr);
-  if (gerr)  {
-    Log::add_err (gerr->message);
-    g_clear_error (&gerr);
-  }
-#else
   if (!gtk_ui_manager_add_ui_from_file (_ui_manager, filename, NULL))
     gtk_ui_manager_add_ui_from_string (_ui_manager, fallback_ui_file, -1, NULL);
-#endif
   g_free (filename);
   g_signal_connect (_ui_manager, "add_widget", G_CALLBACK (add_widget), this);
   add_actions (this, _ui_manager, &prefs);
@@ -527,7 +519,7 @@ GUI :: prompt_user_for_save_path (GtkWindow * parent, const Prefs& prefs)
                                                GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
                                                NULL);
   gtk_dialog_set_default_response (GTK_DIALOG(w), GTK_RESPONSE_ACCEPT);
-  gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(w), prev_path.c_str());
+  gtk_file_chooser_select_filename (GTK_FILE_CHOOSER(w), prev_path.c_str());
   const int response (gtk_dialog_run (GTK_DIALOG(w)));
   std::string path;
   if (response == GTK_RESPONSE_ACCEPT) {
@@ -556,6 +548,79 @@ void GUI :: do_save_articles ()
   }
 }
 
+namespace
+{
+  struct SaveArticlesFromNZB: public Progress::Listener
+  {
+    Data& _data;
+    Queue& _queue;
+    GtkWidget * _root;
+    Prefs& _prefs;
+    ArticleCache& _cache;
+    const Article _article;
+    const std::string _path;
+
+    SaveArticlesFromNZB (Data& d, Queue& q, GtkWidget *r, Prefs& p,
+                         ArticleCache& c, const Article& a, const std::string& path):
+      _data(d), _queue(q), _root(r), _prefs(p), _cache(c), _article(a), _path(path) {}
+
+    static void foreach_part_cb (GMimeObject *o, gpointer self)
+    {
+      static_cast<SaveArticlesFromNZB*>(self)->foreach_part (o);
+    }
+
+    void foreach_part (GMimeObject *o)
+    {
+      if (GMIME_IS_MULTIPART (o))
+      {
+        g_mime_multipart_foreach (GMIME_MULTIPART (o), foreach_part_cb, this);
+      }
+      else if (GMIME_IS_PART(o))
+      {
+        GMimePart * part (GMIME_PART (o));
+        GMimeDataWrapper * wrapper (g_mime_part_get_content_object (part));
+        GMimeStream * mem_stream (g_mime_stream_mem_new ());
+        g_mime_data_wrapper_write_to_stream (wrapper, mem_stream);
+        const GByteArray * buffer (GMIME_STREAM_MEM(mem_stream)->buffer);
+        const StringView nzb ((const char*)buffer->data, buffer->len);
+        Queue::tasks_t tasks;
+        NZB :: tasks_from_nzb_string (nzb, _path, _cache, _data, _data, _data, tasks);
+        if (!tasks.empty())
+          _queue.add_tasks (tasks, Queue::BOTTOM);
+        g_object_unref (mem_stream);
+        g_object_unref (wrapper);
+      }
+    }
+
+    virtual ~SaveArticlesFromNZB() {}
+    
+    virtual void on_progress_finished (Progress&, int status)
+    {
+      if (status == OK) {
+        GMimeMessage * message = _cache.get_message (_article.get_part_mids());
+        g_mime_message_foreach_part (message, foreach_part_cb, this);
+        g_object_unref (message);
+      }
+      delete this;
+    }
+  };
+}
+
+void GUI :: do_save_articles_from_nzb ()
+{
+  const Article* article (_header_pane->get_first_selected_article ());
+  if (article)
+  {
+    const std::string path (GUI :: prompt_user_for_save_path (get_window(_root), _prefs));
+    if (!path.empty())
+    {
+      SaveArticlesFromNZB * listener = new SaveArticlesFromNZB (_data, _queue, _root, _prefs, _cache, *article, path);
+      Task * t = new TaskArticle (_data, _data, *article, _cache, _data, listener);
+      _queue.add_task (t, Queue::TOP);
+    }
+  }
+}
+
 void GUI :: do_print ()
 {
   std::cerr << "FIXME " << LINE_ID << std::endl;
@@ -568,11 +633,22 @@ void GUI :: do_import_tasks ()
 {
   // get a list of files to import
   GtkWidget * dialog = gtk_file_chooser_dialog_new (
-    _("Import NZB File(s)"), get_window(_root),
+    _("Import NZB Files"), get_window(_root),
     GTK_FILE_CHOOSER_ACTION_OPEN,
     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
     GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
     NULL);
+
+  GtkFileFilter * filter = gtk_file_filter_new ();
+  gtk_file_filter_add_pattern (filter, "*.[Nn][Zz][Bb]");
+  gtk_file_filter_set_name (filter, _("NZB Files"));
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(dialog), filter);
+
+  filter = gtk_file_filter_new ();
+  gtk_file_filter_add_pattern (filter, "*");
+  gtk_file_filter_set_name (filter, _("All Files"));
+  gtk_file_chooser_add_filter (GTK_FILE_CHOOSER(dialog), filter);
+
   gtk_file_chooser_set_select_multiple (GTK_FILE_CHOOSER(dialog), true);
   typedef std::vector<std::string> strings_t;
   strings_t filenames;
@@ -694,9 +770,8 @@ void GUI :: do_show_preferences_dialog ()
 }
 void GUI :: do_show_group_preferences_dialog ()
 {
-  const Quark group (_group_pane->get_selection ());
-  if (!group.empty())
-  {
+  const Quark group (_group_pane->get_first_selection ());
+  if (!group.empty()) {
     GroupPrefsDialog * dialog = new GroupPrefsDialog (_data, group, _group_prefs, get_window(_root));
     gtk_widget_show (dialog->root());
   }
@@ -740,7 +815,13 @@ void GUI :: on_progress_finished (Progress & p, int status)
 {
   TaskArticle * ta = dynamic_cast<TaskArticle*>(&p);
   if (status==OK && ta)
+  {
     _body_pane->set_article (ta->get_article());
+
+    // maybe update the tab
+    if (is_action_active ("tabbed-layout"))
+      activate_action ("jump-to-body-tab");
+  }
 }
 
 void GUI :: do_read_selected_article ()
@@ -748,15 +829,10 @@ void GUI :: do_read_selected_article ()
   const Article* article (_header_pane->get_first_selected_article ());
   if (article)
   {
-    Task * t = new TaskArticle (_data, _data, *article, _cache, _data, this);
-    _queue.add_task (t, Queue::TOP);
-
-    // expand its thread in the header pane
     _header_pane->expand_selected ();
 
-    // maybe update the tab
-    if (is_action_active ("tabbed-layout"))
-      activate_action ("jump-to-body-tab");
+    Task * t = new TaskArticle (_data, _data, *article, _cache, _data, this);
+    _queue.add_task (t, Queue::TOP);
   }
 }
 void GUI :: do_download_selected_article ()
@@ -920,7 +996,7 @@ GUI :: score_add (int mode)
 {
   Quark group (_header_pane->get_group());
   if (group.empty())
-    group = _group_pane->get_selection();
+    group = _group_pane->get_first_selection();
 
   Article a;
   const Article* article (_header_pane->get_first_selected_article ());
@@ -1103,7 +1179,7 @@ GUI :: do_post ()
   // add newsgroup...
   Quark group (_header_pane->get_group ());
   if (group.empty())
-    group = _group_pane->get_selection ();
+    group = _group_pane->get_first_selection ();
   if (!group.empty())
     g_mime_message_add_header (message, "Newsgroups", group.c_str());
 
@@ -1386,22 +1462,22 @@ void GUI :: do_show_selected_article_info ()
     char * date = date_maker.get_date_string (a->time_posted);
 
     // article parts
-    std::vector<int> missing_parts;
-    int i = 0;
-    foreach_const (Article::parts_t, a->parts, it) {
-      ++i;
-      if (it->empty())
-        missing_parts.push_back (i);
-    }
+    typedef Parts::number_t number_t;
+    std::set<number_t> missing_parts;
+    const number_t n_parts = a->get_total_part_count ();
+    for (number_t i=1; i<=n_parts; ++i)
+      missing_parts.insert (i);
+    for (Article::part_iterator it(a->pbegin()), e(a->pend()); it!=e; ++it)
+      missing_parts.erase (it.number());
     char msg[512];
     *msg = '\0';
     std::ostringstream s;
-    if (i > 1) {
+    if (n_parts > 1) {
       if (missing_parts.empty())
-        g_snprintf (msg, sizeof(msg), _("This article has all %d parts."), i);
+        g_snprintf (msg, sizeof(msg), _("This article has all %d parts."), (int)n_parts);
       else
-        g_snprintf (msg, sizeof(msg), _("This article is missing %d of its %d parts:"), (int)missing_parts.size(), i);
-      foreach_const (std::vector<int>, missing_parts, it)
+        g_snprintf (msg, sizeof(msg), _("This article is missing %d of its %d parts:"), (int)missing_parts.size(), (int)n_parts);
+      foreach_const (std::set<number_t>, missing_parts, it)
         s << ' ' << *it;
     }
 
@@ -1433,7 +1509,7 @@ void GUI :: do_quit ()
 }
 void GUI :: do_read_selected_group ()
 {
-  const Quark group (_group_pane->get_selection ());
+  const Quark group (_group_pane->get_first_selection ());
 
   // update the titlebar
   std::string s (_("Pan"));
@@ -1444,11 +1520,8 @@ void GUI :: do_read_selected_group ()
   gtk_window_set_title (get_window(_root), s.c_str());
 
   // set the charset encoding based upon that group's default
-  if (!group.empty()) {
-    std::string charset (_group_prefs.get_string (group, "character-encoding", "UTF-8"));
-    charset.insert (0, "charset-");
-    toggle_action (charset.c_str(), true);
-  }
+  if (!group.empty())
+    set_charset (_group_prefs.get_string (group, "character-encoding", "UTF-8"));
 
   // update the header pane
   watch_cursor_on ();
@@ -1484,21 +1557,22 @@ void GUI :: do_read_selected_group ()
 }
 void GUI :: do_mark_selected_groups_read ()
 {
-  const quarks_t group_names (_group_pane->get_full_selection ());
-  foreach_const (quarks_t, group_names, it)
+  const quarks_v group_names (_group_pane->get_full_selection ());
+  foreach_const (quarks_v, group_names, it)
     _data.mark_group_read (*it);
 }
 void GUI :: do_clear_selected_groups ()
 {
-  Quark group (_group_pane->get_selection());
-  _data.group_clear_articles (group);
+  const quarks_v groups (_group_pane->get_full_selection ());
+  foreach_const (quarks_v, groups, it)
+    _data.group_clear_articles (*it);
 }
 
 void GUI :: do_xover_selected_groups ()
 {
-  const quarks_t group_names (_group_pane->get_full_selection ());
+  const quarks_v groups (_group_pane->get_full_selection ());
   const bool mark_read (_prefs.get_flag ("mark-group-read-before-xover", false));
-  foreach_const (quarks_t, group_names, it) {
+  foreach_const (quarks_v, groups, it) {
     if (mark_read)
       _data.mark_group_read (*it);
     _queue.add_task (new TaskXOver (_data, *it, TaskXOver::NEW), Queue::TOP);
@@ -1520,7 +1594,8 @@ void GUI :: do_xover_subscribed_groups ()
 
 void GUI :: do_download_headers ()
 {
-  const quarks_t groups (_group_pane->get_full_selection ());
+  const quarks_v groups_v (_group_pane->get_full_selection ());
+  const quarks_t groups (groups_v.begin(), groups_v.end());
   headers_dialog (_data, _prefs, _queue, groups, get_window(_root));
 }
 
@@ -1538,20 +1613,32 @@ void GUI :: do_refresh_groups ()
 
 void GUI :: do_subscribe_selected_groups ()
 {
-  const quarks_t group_names (_group_pane->get_full_selection ());
-  foreach (quarks_t, group_names, it)
+  const quarks_v group_names (_group_pane->get_full_selection ());
+  foreach_const (quarks_v, group_names, it)
     _data.set_group_subscribed (*it, true);
 }
 void GUI :: do_unsubscribe_selected_groups ()
 {
-  const quarks_t group_names (_group_pane->get_full_selection ());
-  foreach (quarks_t, group_names, it)
+  const quarks_v groups (_group_pane->get_full_selection ());
+  foreach_const (quarks_v, groups, it)
     _data.set_group_subscribed (*it, false);
 }
-void GUI :: do_set_charset (const char * charset)
+void GUI :: do_prompt_for_charset ()
 {
-  _fallback_charset = charset;
-  _body_pane->set_character_encoding (charset);
+  if (_charset.empty())
+      _charset = "UTF-8";
+
+  char * tmp = e_charset_picker_dialog (_("Character Encoding"),
+                                        _("Body Pane Encoding"),
+                                        _charset.c_str(),
+                                        get_window(root()));
+  set_charset (tmp);
+  free (tmp);
+}
+void GUI :: set_charset (const StringView& s)
+{
+  _charset.assign (s.str, s.len);
+  _body_pane->set_character_encoding (_charset.c_str());
 }
 
 /***
