@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-
+#include <memory> 
 #include <config.h>
 #include <signal.h>
 extern "C" {
@@ -27,8 +27,10 @@ extern "C" {
 #include <pan/general/debug.h>
 #include <pan/general/log.h>
 #include <pan/general/file-util.h>
+#include <pan/general/worker-pool.h>
 #include <pan/tasks/socket-impl-gio.h>
 #include <pan/tasks/task-groups.h>
+#include <pan/tasks/task-xover.h>
 #include <pan/tasks/nzb.h>
 #include <pan/data-impl/data-impl.h>
 #include <pan/icons/pan-pixbufs.h>
@@ -158,6 +160,30 @@ namespace
     gtk_widget_destroy (GTK_WIDGET(window));
   }
 
+  /** Queue:::Listener that quits Pan via mainloop_exit()
+      when on_queue_size_changed() say the queue is empty.
+      See bugzilla bug #424248. */
+  struct PanKiller : public Queue::Listener
+  {
+    PanKiller(Queue & q) : q(q) { q.add_listener(this); }
+    ~PanKiller() { q.remove_listener(this); }
+
+    /** Method from Queue::Listener interface: quits program on zero sized Q*/
+    void on_queue_size_changed (Queue&, int active, int total) 
+      {  if (!active && !total) mainloop_quit();  }
+
+    // all below methods from Queue::Listener interface are noops
+    void on_queue_task_active_changed (Queue&, Task&, bool) {}
+    void on_queue_tasks_added (Queue&, int , int ) {}
+    void on_queue_task_removed (Queue&, Task&, int) {}
+    void on_queue_task_moved (Queue&, Task&, int, int) {}
+    void on_queue_connection_count_changed (Queue&, int) {}
+    void on_queue_online_changed (Queue&, bool) {}
+    void on_queue_error (Queue&, const StringView&) {}
+  private:
+    Queue & q;
+  };
+
   void usage ()
   {
     std::cerr << "Pan " << VERSION << "\n\n" <<
@@ -167,6 +193,7 @@ _("General Options\n"
 "URL Options\n"
 "  news:message-id          Show the specified article.\n"
 "  news:group.name          Show the specified newsgroup.\n"
+"  headers:group.name       Download new headers for the specified newsgroup.\n"
 "  --no-gui                 On news:message-id, dump the article to stdout.\n"
 "\n"
 "NZB Batch Options\n"
@@ -184,11 +211,11 @@ main (int argc, char *argv[])
   textdomain (GETTEXT_PACKAGE);
 
   g_thread_init (0);
-  gtk_init (&argc, &argv);
   g_mime_init (GMIME_INIT_FLAG_UTF8);
 
   bool gui(true), nzb(false);
   std::string url;
+  std::string groups;
   std::string nzb_output_path;
   typedef std::vector<std::string> strings_t;
   strings_t nzb_files;
@@ -196,13 +223,16 @@ main (int argc, char *argv[])
   for (int i=1; i<argc; ++i)
   {
     const char * tok (argv[i]);
-    if (!memcmp(tok,"news:", 5))
+    if (!memcmp(tok,"headers:", 8))
+      groups = tok+8;
+    else if (!memcmp(tok,"news:", 5))
       url = tok;
     else if (!strcmp(tok,"--no-gui") || !strcmp(tok,"--nogui"))
       gui = false;
-    else if (!strcmp (tok, "--debug"))
-      _debug_flag = true;
-    else if (!strcmp (tok, "--nzb"))
+    else if (!strcmp (tok, "--debug")) { // do --debug --debug for verbose debug
+      if (_debug_flag) _debug_verbose_flag = true;
+      else _debug_flag = true;
+    } else if (!strcmp (tok, "--nzb"))
       nzb = true;
     else if (!strcmp (tok, "--version"))
       { std::cerr << "Pan " << VERSION << '\n'; return 0; }
@@ -218,7 +248,10 @@ main (int argc, char *argv[])
     }
   }
 
-  if (!gui && nzb_files.empty() && url.empty()) {
+  if (gui)
+    gtk_init (&argc, &argv);
+
+  if (!gui && nzb_files.empty() && url.empty() && groups.empty()) {
     std::cerr << _("Error: --no-gui used without nzb files or news:message-id.") << std::endl;
     return 0;
   }
@@ -244,24 +277,35 @@ main (int argc, char *argv[])
     }
 
     // instantiate the queue...
+    WorkerPool worker_pool (4, true);
     GIOChannelSocket::Creator socket_creator;
-    Queue queue (data, data, &socket_creator, prefs.get_flag("work-online",true));
+    Queue queue (data, data, &socket_creator, worker_pool, prefs.get_flag("work-online", true));
     g_timeout_add (5000, queue_upkeep_timer_cb, &queue);
 
-    if (nzb)
+    if (nzb || !groups.empty())
     {
-      // if no save path was specified, either prompt for one or
-      // use the user's home directory as a fallback.
-      if (nzb_output_path.empty() && gui)
-        nzb_output_path = GUI::prompt_user_for_save_path (NULL, prefs);
-      if (nzb_output_path.empty()) // user pressed `cancel' when prompted
-        return 0;
+      StringView tok, v(groups);
+      while (v.pop_token(tok,','))
+        queue.add_task (new TaskXOver (data, tok, TaskXOver::NEW), Queue::BOTTOM);
 
-      // load the nzb files...
-      std::vector<Task*> tasks;
-      foreach_const (strings_t, nzb_files, it)
-        NZB :: tasks_from_nzb_file (*it, nzb_output_path, cache, data, data, data, tasks);
-      queue.add_tasks (tasks, Queue::BOTTOM);
+      if (nzb)
+      {
+        // if no save path was specified, either prompt for one or
+        // use the user's home directory as a fallback.
+        if (nzb_output_path.empty() && gui)
+          nzb_output_path = GUI::prompt_user_for_save_path (NULL, prefs);
+        if (nzb_output_path.empty()) // user pressed `cancel' when prompted
+          return 0;
+
+        // load the nzb files...
+        std::vector<Task*> tasks;
+        foreach_const (strings_t, nzb_files, it)
+          NZB :: tasks_from_nzb_file (*it, nzb_output_path, cache, data, data, data, tasks);
+        queue.add_tasks (tasks, Queue::BOTTOM);
+      }
+
+      // iff non-gui mode, contains a PanKiller ptr to quit pan on queue empty
+      std::auto_ptr<PanKiller> killer;
 
       // don't open the full-blown Pan, just act as a nzb client,
       // with a gui or without.
@@ -273,6 +317,8 @@ main (int argc, char *argv[])
         g_signal_connect (G_OBJECT(w), "delete-event", G_CALLBACK(delete_event_cb), 0);
       } else {
         nongui_gmainloop = g_main_loop_new (NULL, false);
+        // create a PanKiller object -- which quits pan when the queue is done
+        killer.reset(new PanKiller(queue));
       }
       register_shutdown_signals ();
       mainloop ();
@@ -294,6 +340,8 @@ main (int argc, char *argv[])
       run_pan_in_window (cache, data, queue, prefs, group_prefs, GTK_WINDOW(window));
     }
 
+    worker_pool.cancel_all_silently ();
+
     if (prefs.get_flag("clear-article-cache-on-shutdown", false))
       cache.clear ();
   }
