@@ -53,16 +53,16 @@ namespace pan
       void rot13_selection ();
       void wrap_body ();
       void spawn_editor ();
-      void manage_editors ();
       void manage_profiles ();
-      void set_editor_command (const StringView&);
       void set_charset (const StringView&);
       void apply_profile ();
       void save_draft ();
       void open_draft ();
+      void prompt_for_charset ();
       void send_now ();
       void close_window ();
       void set_wrap_mode (bool wrap);
+      void set_always_run_editor (bool);
 
     private:
       void done_sending_message (GMimeMessage*, bool);
@@ -70,7 +70,6 @@ namespace pan
       bool maybe_post_message (GMimeMessage*);
 
     private:
-      void add_editors_to_menu ();
       void update_widgetry ();
       void update_profile_combobox ();
       void populate_from_message (GMimeMessage*);
@@ -102,7 +101,6 @@ namespace pan
       GMimeMessage * _message;
       std::string _charset;
       TextMassager _tm;
-      std::set<unsigned int> _editor_ui_ids;
       GtkUIManager * _uim;
       GtkActionGroup * _agroup;
       std::string _signature;
@@ -117,8 +115,6 @@ namespace pan
 
     private:
       void add_actions (GtkWidget* box);
-      void add_editor_list ();
-      void add_charset_list ();
       void apply_profile_to_body ();
       void apply_profile_to_headers ();
       enum Mode { DRAFTING, POSTING };
@@ -132,6 +128,7 @@ namespace pan
       GtkWidget* create_extras_tab ();
 
     private:
+      std::string utf8ize (const StringView&) const;
       std::string get_body () const;
       gulong body_view_realized_handler;
       static void body_view_realized_cb (GtkWidget*, gpointer);
@@ -143,8 +140,6 @@ namespace pan
       unsigned int _group_entry_changed_idle_tag;
       static gboolean group_entry_changed_idle (gpointer);
       static void group_entry_changed_cb (GtkEditable*, gpointer);
-      static void charset_selected_cb_static (GtkRadioAction*, GtkRadioAction*, gpointer);
-      void charset_selected_cb (const char * charset);
 
     public:
       void set_spellcheck_enabled (bool);
