@@ -277,11 +277,11 @@ NNTP :: on_socket_response (Socket * sock, const StringView& line_in)
 
    bool more;
    switch (state) {
-      case CMD_FAIL: fire_done_func (COMMAND_FAILED, line); more = false; break;
+      case CMD_FAIL: fire_done_func (ERR_COMMAND, line); more = false; break;
       case CMD_DONE: if (_commands.empty()) fire_done_func (OK, line); more = false; break;
       case CMD_MORE: more = true; break; // keep listining for more on this command
       case CMD_NEXT: more = false; break; // no more responses on this command; wait for next...
-      case CMD_RETRY: fire_done_func (NETWORK_FAILED, line); more = false; break;
+      case CMD_RETRY: fire_done_func (ERR_NETWORK, line); more = false; break;
       default: abort(); break;
    }
    return more;
@@ -290,14 +290,14 @@ NNTP :: on_socket_response (Socket * sock, const StringView& line_in)
 void
 NNTP :: on_socket_abort (Socket * sock)
 {
-   fire_done_func (NETWORK_FAILED, StringView());
+   fire_done_func (ERR_NETWORK, StringView());
 }
 
 void
 NNTP :: on_socket_error (Socket * sock)
 {
    _socket_error = true;
-   fire_done_func (NETWORK_FAILED, StringView());
+   fire_done_func (ERR_NETWORK, StringView());
 }
 
 namespace
