@@ -26,9 +26,12 @@ extern "C" {
   #include <glib.h>
 }
 #include <pan/general/debug.h>
+#include <pan/general/file-util.h>
+#include <pan/general/foreach.h>
+#include <pan/general/log.h>
 #include <pan/general/quark.h>
 #include <pan/general/string-view.h>
-#include <pan/general/foreach.h>
+#include <pan/general/utf8-utils.h>
 #include "nzb.h"
 #include "task-article.h"
 
@@ -37,7 +40,6 @@ using namespace pan;
 namespace
 {
   typedef std::vector<Task*> tasks_t;
-  typedef std::map<unsigned int,Article::Part> number_to_part_t;
 
   struct MyContext
   {
@@ -45,7 +47,7 @@ namespace
     std::string text;
     std::string path;
     Article a;
-    number_to_part_t parts;
+    PartBatch parts;
     tasks_t tasks;
     ArticleCache& cache;
     ArticleRead& read;
@@ -63,7 +65,6 @@ namespace
       groups.clear ();
       text.clear ();
       path.clear ();
-      parts.clear ();
       a.clear ();
       bytes = 0;
       number = 0;
@@ -112,22 +113,19 @@ namespace
 
     else if (!strcmp(element_name, "segment") && mc.number && !mc.text.empty()) {
       const std::string mid ("<" + mc.text + ">");
-      if (mc.a.message_id.empty())
-          mc.a.message_id = mid;
-      Article::Part& part (mc.parts[mc.number]);
-      part.bytes = mc.bytes;
-      part.set_message_id (mc.a.message_id, mid);
+      if (mc.a.message_id.empty()) {
+        mc.a.message_id = mid;
+        mc.parts.init (mid);
+      }
+      mc.parts.add_part (mc.number, mid, mc.bytes);
     }
 
     else if (!strcmp(element_name,"path"))
       mc.path = mc.text;
 
-    else if (!mc.parts.empty() && !strcmp (element_name, "file"))
+    else if (!strcmp (element_name, "file"))
     {
-      // populate mc.a.parts
-      mc.a.set_part_count (mc.parts.rbegin()->first);
-      foreach (number_to_part_t, mc.parts, it)
-        mc.a.get_part(it->first).swap (it->second);
+      mc.a.set_parts (mc.parts);
 
       foreach_const (quarks_t, mc.groups, git) {
         quarks_t servers;
@@ -142,7 +140,7 @@ namespace
 
   void text (GMarkupParseContext *context,
              const gchar         *text,
-             gsize                text_len,  
+             gsize                text_len,
              gpointer             user_data,
              GError             **error)
   {
@@ -151,7 +149,7 @@ namespace
 }
 
 void
-NZB :: tasks_from_nzb_string (const StringView      & nzb,
+NZB :: tasks_from_nzb_string (const StringView      & nzb_in,
                               const StringView      & save_path,
                               ArticleCache          & cache,
                               ArticleRead           & read,
@@ -159,6 +157,7 @@ NZB :: tasks_from_nzb_string (const StringView      & nzb,
                               const GroupServer     & gs,
                               std::vector<Task*>    & appendme)
 {
+  const std::string nzb (clean_utf8 (nzb_in));
   MyContext mc (cache, read, ranks, gs, save_path);
   GMarkupParser p;
   p.start_element = start_element;
@@ -169,8 +168,10 @@ NZB :: tasks_from_nzb_string (const StringView      & nzb,
   GMarkupParseContext* c (
     g_markup_parse_context_new (&p, (GMarkupParseFlags)0, &mc, 0));
   GError * gerr (0);
-  if (g_markup_parse_context_parse (c, nzb.str, nzb.len, &gerr)) {
-    // FIXME
+  g_markup_parse_context_parse (c, nzb.c_str(), nzb.size(), &gerr);
+  if (gerr) {
+    Log::add_urgent (gerr->message);
+    g_error_free (gerr);
   }
   g_markup_parse_context_free (c);
   appendme.insert (appendme.end(), mc.tasks.begin(), mc.tasks.end());
@@ -185,11 +186,9 @@ NZB :: tasks_from_nzb_file (const StringView      & filename,
                             const GroupServer     & gs,
                             std::vector<Task*>    & appendme)
 {
-  gchar * txt (0);
-  gsize len (0);
-  if (g_file_get_contents (filename.to_string().c_str(), &txt, &len, 0))
-    tasks_from_nzb_string (StringView(txt,len), save_path, c, r, ranks, gs, appendme);
-  g_free (txt);
+  std::string nzb;
+  if (file :: get_text_file_contents (filename, nzb))
+    tasks_from_nzb_string (nzb, save_path, c, r, ranks, gs, appendme);
 }
 
 namespace
@@ -220,7 +219,7 @@ NZB :: nzb_to_xml (std::ostream             & out,
                    const std::vector<Task*> & tasks)
 {
   int depth (0);
-  
+
   out << "<?xml version=\"1.0\" encoding=\"utf-8\" ?>\n"
       << "<!DOCTYPE nzb PUBLIC \"-//newzBin//DTD NZB 1.0//EN\" \"http://www.newzbin.com/DTD/nzb/nzb-1.0.dtd\">\n"
       << indent(depth++)
@@ -249,7 +248,7 @@ NZB :: nzb_to_xml (std::ostream             & out,
       escaped (out, path.to_view());
       out << "</path>\n";
     }
-    
+
     // what groups was this crossposted in?
     quarks_t groups;
     foreach_const (Xref, a.xref, xit)
@@ -261,17 +260,11 @@ NZB :: nzb_to_xml (std::ostream             & out,
 
     // now for the parts...
     out << indent(depth++) << "<segments>\n";
-    int part_number (0);
-    foreach_const (Article::parts_t, a.parts, pit)
+    for (Article::part_iterator it(a.pbegin()), end(a.pend()); it!=end; ++it)
     {
-      ++part_number;
-
-      // incomplete multipart...
-      if (pit->empty())
-        continue;
+      std::string mid = it.mid ();
 
       // remove the surrounding < > as per nzb spec
-      std::string mid (pit->get_message_id (a.message_id));
       if (mid.size()>=2 && mid[0]=='<') {
         mid.erase (0, 1);
         mid.resize (mid.size()-1);
@@ -279,8 +272,8 @@ NZB :: nzb_to_xml (std::ostream             & out,
 
       // serialize this part
       out << indent(depth)
-          << "<segment" << " bytes=\"" << pit->bytes << '"'
-                        << " number=\"" << part_number << '"'
+          << "<segment" << " bytes=\"" << it.bytes() << '"'
+                        << " number=\"" << it.number() << '"'
                         << ">";
       escaped(out, mid);
       out  << "</segment>\n";
