@@ -57,7 +57,7 @@
 #include <fptools.h>
 #include <uustring.h>
 
-char * uuscan_id = "$Id: uuscan.c 62 2006-11-17 14:43:16Z csk $";
+char * uuscan_id = "$Id: uuscan.c 300 2007-05-26 16:28:55Z csk $";
 
 /*
  * Header fields we recognize as such. See RFC822. We add "From ",
@@ -170,7 +170,7 @@ ScanHeaderLine (FILE *datei, char *initial)
 {
   char *ptr=uuscan_shlline;
   char *ptr2, *p1, *p2, *p3;
-  int llength, c;
+  int llength;
   long curpos;
   int hadcr;
 
@@ -202,7 +202,7 @@ ScanHeaderLine (FILE *datei, char *initial)
   }
 
   while (!feof (datei)) {
-    c = fgetc (datei);
+    char c = fgetc (datei);
     if (feof (datei))
       break;
 
@@ -309,6 +309,7 @@ ScanHeaderLine (FILE *datei, char *initial)
 	}
 
 	if (*p3 == '?' && *(p3+1) == '=') {
+          char c = -1;
 	  /*
 	   * Alright, we've found an RFC 1522 header field
 	   */
@@ -626,6 +627,7 @@ ScanData (FILE *datei, char *fname, int *errcode,
   long preheaders=0, oldposition;
   long yefilesize=0, yepartends=0;
   size_t dcc, bhopc;
+  int oldcheckheaders;
 
   *errcode = UURET_OK;
   (void) UUDecodeLine (NULL, NULL, 0);          /* init */
@@ -1013,10 +1015,13 @@ ScanData (FILE *datei, char *fname, int *errcode,
 	result->begin = 1;
       }
 
+      /* don't scan for headers inside a yenc block */
+      oldcheckheaders = checkheaders;
+      checkheaders = 0;
+
       /*
        * Don't want auto-detection
        */
-
       result->uudet = YENC_ENCODED;
       continue;
     }
@@ -1030,6 +1035,9 @@ ScanData (FILE *datei, char *fname, int *errcode,
       if (!uu_more_mime)
 	break;
 #endif
+
+      /* restore the checkheaders state we had before =ybegin */
+      checkheaders = oldcheckheaders;
       continue;
     }
 
